@@ -2,7 +2,7 @@
 # Licensed Materials - Property of IBM
 # 5737-I32
 #
-# (C) Copyright IBM Corp. 2019
+# (C) Copyright IBM Corp. 2019, 2020
 #
 # US Government Users Restricted Rights - Use, duplication, or
 # disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
@@ -15,7 +15,7 @@ import logging
 import subprocess
 from subprocess import check_call, CalledProcessError
 import tempfile
-from io import open
+from io import open, StringIO
 from re import match
 from functools import partial
 from urllib.parse import urljoin
@@ -27,6 +27,11 @@ from .util.aes_cipher import AesCipher
 
 ENCODING = 'utf-8'
 
+MAX_POLL_INTERVAL = 86400000
+SESSION_TIMEOUT_MS = 60000
+
+DEFAULT_SSH_KEY_LOCATION = '/gpfs/gpfs0/connections/scale/id_rsa'
+
 class ApplicationBase():
     """Application SDK for registration and communication with Spectrum Discover.
 
@@ -51,6 +56,9 @@ class ApplicationBase():
 
     MAX_POLL_INTERVAL .......... kafka config for max.poll.interval.ms
                                  - default: 86400000
+
+    SSH_KEY_LOCATION ........... Full path to the private ssh key - id_rsa file
+                                 - default: None
     """
 
     def __init__(self, reg_info):
@@ -73,7 +81,9 @@ class ApplicationBase():
         self.application_name = env('APPLICATION_NAME', 'sd_sample_application')
 
         self.is_kube = os.environ.get('KUBERNETES_SERVICE_HOST') is not None
+        self.is_docker = os.environ.get('IS_DOCKER_CONTAINER', False)
 
+        self.cipherkey = None
         self.cipher = None
 
         # The user account assigned to this application
@@ -155,9 +165,6 @@ class ApplicationBase():
         self.logger.info("Application user: %s", self.application_user)
         self.logger.info("Certificates directory: %s", self.certificates_dir)
 
-        if not self.application_user:
-            raise Exception("Authentication requires APPLICATION_USER and APPLICATION_USER_PASSWORD")
-
         # can be set by apps when starting if they wish to overwrite existing registration info
         self.update_registration = False
 
@@ -360,11 +367,12 @@ class ApplicationBase():
         self.kafka_producer = Producer(p_conf)
 
         # Instantiate consumer
-        max_poll_interval = int(os.environ.get('MAX_POLL_INTERVAL', 86400000))
+        max_poll_interval = int(os.environ.get('MAX_POLL_INTERVAL', MAX_POLL_INTERVAL))
+        session_timeout_ms = int(os.environ.get('SESSION_TIMEOUT_MS', SESSION_TIMEOUT_MS))
         c_conf = {
             'bootstrap.servers': '%s' % self.kafka_host,
             'group.id': 'myagent_grp',
-            'session.timeout.ms': 6000,
+            'session.timeout.ms': session_timeout_ms,
             'max.poll.interval.ms': max_poll_interval,
             'default.topic.config': {'auto.offset.reset': 'smallest'},
             'ssl.certificate.location': self.kafka_client_cert,
@@ -425,13 +433,14 @@ class ApplicationBase():
 
             response = requests.get(url=self.connmgr_url, verify=False, headers=headers, auth=auth)
 
-            cipherkey = os.environ.get('CIPHER_KEY', None)
-            if cipherkey:
-                self.cipher = AesCipher(cipherkey)
+            self.cipherkey = os.environ.get('CIPHER_KEY', None)
+            if self.cipherkey:
+                self.cipher = AesCipher(self.cipherkey)
             else:
                 cipherkey_response = requests.get(url=self.cipher_url, verify=False, headers=headers, auth=auth)
                 if cipherkey_response.ok:
-                    self.cipher = AesCipher(cipherkey_response.json()['cipher_key'])
+                    self.cipherkey = cipherkey_response.json()['cipher_key']
+                    self.cipher = AesCipher(self.cipherkey)
                 else:
                     self.logger.warning("Cipher key was not available. This may affect cos and scale connections")
 
@@ -558,13 +567,8 @@ class ApplicationBase():
         if not conn['online']:
             self.logger.info('Skipping creation of offline scale connection: %s', conn['host'])
             return
-        if not conn['password']:
-            self.logger.warning('Skipping creation of scale connection: %s due to missing password', conn['host'])
-            return
 
         try:
-            xport = paramiko.Transport(conn['host'])
-
             local_conn = False
             try:
                 proc = subprocess.Popen(['/usr/lpp/mmfs/bin/mmlscluster'], stdout=subprocess.PIPE)
@@ -579,7 +583,27 @@ class ApplicationBase():
                 self.logger.info('Successfully created local scale connection for: %s', conn['name'])
                 return
 
-            xport.connect(username=conn['user'], password=self.cipher.decrypt(conn['password']))
+            xport = paramiko.Transport(conn['host'])
+
+            additional_info = json.loads(conn['additional_info'])
+
+            if 'auth_type' in additional_info:  # 2.0.3.1+
+                if additional_info['auth_type'] == 'password':
+                    xport.connect(username=conn['user'], password=self.cipher.decrypt(conn['password']))
+                else:
+                    pkey = paramiko.RSAKey.from_private_key(StringIO(additional_info['privkey']), self.cipherkey)
+                    xport.connect(username=conn['user'], pkey=pkey)
+            else:  # <= 2.0.3
+                ssh_key = '/keys/id_rsa' if self.is_kube or self.is_docker else os.environ.get('SSH_KEY_LOCATION', DEFAULT_SSH_KEY_LOCATION)
+
+                if os.path.exists(ssh_key):
+                    pkey = paramiko.RSAKey.from_private_key_file(ssh_key)
+                    xport.connect(username=conn['user'], pkey=pkey)
+                else:
+                    self.logger.error('Could not find ssh key file for connection: %s. '
+                                      'You may need to define the \'SSH_KEY_LOCATION\' environment variable.', conn['name'])
+                    return
+
             sftp = paramiko.SFTPClient.from_transport(xport)
             if sftp:
                 self.connections[(conn['datasource']), conn['cluster']] = ('Spectrum Scale', sftp)
@@ -591,34 +615,22 @@ class ApplicationBase():
 
     def mount_smb(self, conn, local_mount):
         """Mount the SMB file system."""
-        if not conn['host']:
-            self.logger.warning('Host not defined so cannot create SMB mount.')
-            return
-        if not conn['user']:
-            self.logger.warning('Could not retrieve SMB user.')
-            return
-        if not conn['password']:
-            self.logger.warning('Could not retrieve password for SMB mount.')
-            return
-        if not conn['mount_point']:
-            self.logger.warning('Could not retrieve smb export path.')
-            return
-        if not local_mount:
-            self.logger.warning('Host mount path could not be created.')
-            return
+        try:
+            host = conn['host']
+            password = self.cipher.decrypt(conn['password'])
+            export_path = conn['mount_point']
 
-        host = conn['host']
-        password = self.cipher.decrypt(conn['password'])
-        export_path = conn['mount_point']
-
-        if '\\' in conn['user']:
-            (domain, user) = conn['user'].split('\\')
-        elif '/' in conn['user']:
-            (domain, user) = conn['user'].split('/')
-        elif '@' in conn['user']:
-            (user, domain) = conn['user'].split('@')
-        else:
-            (domain, user) = ('', conn['user'])
+            if '\\' in conn['user']:
+                (domain, user) = conn['user'].split('\\')
+            elif '/' in conn['user']:
+                (domain, user) = conn['user'].split('/')
+            elif '@' in conn['user']:
+                (user, domain) = conn['user'].split('@')
+            else:
+                (domain, user) = ('', conn['user'])
+        except KeyError as ke:
+            self.logger.error('Skipping creation of SMB connection: %s. %s is not defined.', conn['name'], str(ke))
+            return False
 
         cmd = f'mount -t cifs \'{export_path}\' {local_mount} -o user=\'{user}\' -o password=\'{password}\' -o ro'
         if domain:
@@ -626,9 +638,14 @@ class ApplicationBase():
 
         try:
             check_call(cmd, shell=True)
-        except CalledProcessError:
-            # Not fatal, this might not be an active connection
-            self.logger.warning('Failed to mount SMB export %s', host)
+            return True
+        except CalledProcessError as cpe:
+            # Failed unless rc=32 which means already mounted
+            if cpe.returncode == 32 and 'Device or resource busy' in str(cpe):
+                self.logger.info("SMB connection %s already mounted.", conn['name'])
+                return True
+            self.logger.warning('Failed to mount SMB export %s for connection %s', export_path, conn['name'])
+            return False
 
     def create_smb_connection(self, conn):
         """Create a SMB connection for retrieving docs using a cifs mount."""
@@ -639,7 +656,10 @@ class ApplicationBase():
         conn['additional_info'] = {}
         conn['additional_info']['local_mount'] = local_mount
 
-        self.mount_smb(conn, local_mount)
+        mounted = self.mount_smb(conn, local_mount)
+        if not mounted:
+            return
+
         self.connections[(conn['datasource']), conn['cluster']] = ('SMB/CIFS', conn)
         self.logger.info('Successfully created smb connection for: %s', conn['name'])
 
